// cbi and sbi are standard (AVR) methods for setting, 
// or clearing, bits in PORT (and other) variables.
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif 
//----------------------------------------------------------------------------------------------

word avrRawTemp(word samples)
  { // gets raw reading in the range 0 to 1023 from internal temperature sensor 
    // using internal volt 1.1 voltage ref
    // Should return ~325-400 for normal enviroment temperatures
    unsigned long RawSum=0;                         // used to sum samples for averaging 
    word RawTemp=0;                                 // used to accumalate 10 bit ADC readings 
    word test=0;                                    // used to count samples
    byte exp=0;                                     // samples = 2 to the exp power, used as shift operand
    unsigned long Start=micros();                   // this was used for benchmark timeing

    // set system clock devisor to 128
    // 16 MHz / 128 = 125 KHz, inside the desired 50-200 KHz range.
    sbi(ADCSRA, ADPS2);                            // bit 2 of ADCSRA, system clock devisor
    sbi(ADCSRA, ADPS1);                            // bit 1 of ADCSRA, system clock devisor
    sbi(ADCSRA, ADPS0);                            // bit 0 of ADCSRA, system clock devisor
    cbi(ADCSRA, ADATE);                            // bit 5 of ADCSRA, disable auto trigger mode
    sbi(ADCSRA, ADEN);                             // bit 7 of ADCSRA, enable ADC 
    
    // turn on internal reference, right-shift ADC buffer,ADC channel = internal temp sensor
    ADMUX = B11001000;                             // High Nibble: 1100 = internal 1.1 Vref 
                                                   //  Low Nibble: 1000 = ADC chn 8 Temp sensor
    delay(10);  			            // wait a bit for the analog reference to stabilize
    
    // as "C" lacks an expodential or power function (or operator)
    // we must use a resort to loops to calculate the binary exponent
    while (samples>1)    { samples /=2; exp++;}     // calculate power of 2
    samples=1;                                      // make sure samples = 1 (not 0)
    while (test++ < exp) { samples *=2;}            // set samples value to power of two
    
    test=0;                                         // reset test because we have abused it
    while (test++ < samples)                        // oversampling loop (for averaging) 
      { ADCSRA |= _BV(ADSC); 	                  // start the conversion by setting ADSC=1                               
        while (bit_is_set(ADCSRA, ADSC));           // ADSC is cleared when the conversion finishes
        RawTemp  = (ADCL | (ADCH << 8));            // get the ADC reading (low byte first)
        RawSum  += RawTemp;                         // accumalate 10 bit ADC value
      }
    Time=micros()-Start;                            // record benchmark time 
    return ((RawSum)>>exp);                         // averag by shifting bit position, 
                                                    // LSBs lost 
  }
 
//----------------------------------------------------------------------------------------------
word avrBandGap1(word samples)
  { // gets raw reading in the range 0 to 1023 from internal Band Gap reference 
    // using internal volt 1.1 voltage ref
    // Should return 1023
    unsigned long RawSum=0;
    word RawTemp=0;
    word test=0;
    byte exp=0;
    unsigned long Start=micros();

    sbi(ADCSRA, ADPS2);
    sbi(ADCSRA, ADPS1);
    sbi(ADCSRA, ADPS0);
    cbi(ADCSRA, ADATE);
    sbi(ADCSRA, ADEN);
    
    ADMUX = B11001110;			    	    // High Nibble: 1100 = internal 1.1 Vref 
    delay(10);                                      //  Low Nibble: 1110 = 1.1V BandGap (Vbg)
    
    while (samples>1)    { samples /=2; exp++;}
    samples=1;
    while (test++ < exp) { samples *=2;}
    
    test=0;
    while (test++ < samples)
      { ADCSRA |= _BV(ADSC);
        while (bit_is_set(ADCSRA, ADSC));
        RawTemp  = (ADCL | (ADCH << 8));
        RawSum  += RawTemp;
        RawTemp=0;
      }
    Time=micros()-Start;
    return ((RawSum)>>exp);
  }
  
//-------------------------------------------------------------------------------------------
word avrBandGap2(word samples)
  { // gets raw reading in the range 0 to 1023 from internal Band Gap reference 
    // using internal volt ref AVCC= 5 voltage ref
    // should return ~225
    unsigned long RawSum=0;
    word RawTemp=0;
    word test=0;
    byte exp=0;
    unsigned long Start=micros();
    
    sbi(ADCSRA, ADPS2);
    sbi(ADCSRA, ADPS1);
    sbi(ADCSRA, ADPS0);
    cbi(ADCSRA, ADATE);
    sbi(ADCSRA, ADEN);
    
    ADMUX = B01001110;                           // High Nibble: 0100 = internal 5.0 Vref 
    delay(10);  			            	//  Low Nibble: 1110 = 1.1V BandGap (Vbg)
    
    while (samples>1)    { samples /=2; exp++;}
    samples=1;
    while (test++ < exp) { samples *=2;}
    
    test=0;
    while (test++ < samples)
      { ADCSRA |= _BV(ADSC);
        while (bit_is_set(ADCSRA, ADSC));
        RawTemp  = (ADCL | (ADCH << 8));
        RawSum  += RawTemp;
      }
    Time=micros()-Start;
    return ((RawSum)>>exp);
  }

//----------------------------------------------------------------------------------------------
word avrGround1(word samples)
  { // gets raw reading in the range 0 to 1023 from from internal ground reference 
    // using internal volt 1.1 voltage ref
    // This should return zero 
    unsigned long RawSum=0;
    word RawTemp=0;
    word test=0;
    byte exp=0;
    unsigned long Start=micros();
    
    sbi(ADCSRA, ADPS2);
    sbi(ADCSRA, ADPS1);
    sbi(ADCSRA, ADPS0);
    cbi(ADCSRA, ADATE);
    sbi(ADCSRA, ADEN);
    
    ADMUX = B11001111;				    // High Nibble: 1100 = internal 1.1 Vref 
    delay(10);  			            //  Low Nibble: 1111 = Internal Ground Ref
    
    while (samples>1)    { samples /=2; exp++;}
    samples=1;
    while (test++ < exp) { samples *=2;}
    
    test=0;
    while (test++ < samples)
      { ADCSRA |= _BV(ADSC);
        while (bit_is_set(ADCSRA, ADSC));
        RawTemp = (ADCL | (ADCH << 8));
        RawSum += RawTemp;
      }
    Time=micros()-Start;
    return ((RawSum)>>exp);
  }


//--------------------------------------------------------------------------------------------

word avrGround2(word samples)
  { // gets raw reading in the range 0 to 1023 from from internal ground reference 
    // using internal volt ref AVCC= 5 voltage ref
    // This should return zero 
    unsigned long RawSum=0;
    word RawTemp=0;
    word test=0;
    byte exp=0;
    unsigned long Start=micros();
    
    sbi(ADCSRA, ADPS2);
    sbi(ADCSRA, ADPS1);
    sbi(ADCSRA, ADPS0);
    cbi(ADCSRA, ADATE);
    sbi(ADCSRA, ADEN);
    
    ADMUX = B01001111;					// High Nibble: 0100 = internal 5.0 Vref
    delay(10);  			                //  Low Nibble: 1111 = Internal Ground Ref
    
    while (samples>1)    { samples /=2; exp++;}
    samples=1;
    while (test++ < exp) { samples *=2;}
    
    test=0;
    while (test++ < samples)
      { ADCSRA |= _BV(ADSC);
        while (bit_is_set(ADCSRA, ADSC));
        RawTemp = (ADCL | (ADCH << 8));
        RawSum += RawTemp;
      }
    Time=micros()-Start;
    return ((RawSum)>>exp);
  }


