word avrRawTemp(word samples)
  { /* samples: the number of samples to average                   */
    /* this number will be reduced to a power of 2!                */
    /* return: degrees Kelvin * 1/4, range 0 to 4096               */
    /* each sample has 16 ADC reads for the 12 bit virtual ADC     */
    /* REF: Atmel document number AVR121.pdf                       */
    //-------------------------------------------------------------//
    /* on 16Mhz ATmega328 512 samples requires just under 1 second */
    /* 16 samples (16*16=256) gives fairly consistent results      */
    /*    on a steady-state system in under 40K microseconds       */
    //-------------------------------------------------------------//
    unsigned long RawSum=0;                         // used to sum samples for averaging 
    word RawTemp=0;                                 // used to accumalate 10 bit ADC readings 
    word test=0;                                    // used to count samples
    byte exp=0;                                     // samples = 2 to the exp power, used as shift operand
    byte k=0;                                       // counter for ADC reads
    unsigned long Start=micros();                   // this was used for benchmark timeing
    
    // turn on internal reference, right-shift ADC buffer,ADC channel = internal temp sensor
    ADMUX = 0xC8;
    delay(10);  			            // wait a bit for the analog reference to stabilize
    
    // as "C" lacks an expodential or power function (or operator)
    // we must use a resort to loops to calculate the binary exponent
    while (samples>1)    { samples /=2; exp++;}     // calculate power of 2
    samples=1;                                      // make sure samples = 1 (not 0)
    while (test++ < exp) { samples *=2;}            // set samples value to power of two
    
    test=0;                                         // reset test because we have abused it
    while (test++ < samples)                        // oversampling loop (for averaging) 
          {   for (k=0; k<16; k++) 	            // virtual ADC loop, 16 consecutiive readings
                { ADCSRA |= _BV(ADSC); 	            // start the conversion                                
                  while (bit_is_set(ADCSRA, ADSC)); // ADSC is cleared when the conversion finishes
                  RawTemp += (ADCL | (ADCH << 8));  // accumalate the reading (low byte first)
                }
              RawSum += (RawTemp >>2);              // accumalate virtual 12 bit ADC value
              RawTemp=0;                            // zero ADC accumalator for the next sequence
          }
    Time=micros()-Start;                            // record benchmark time 
    return ((RawSum)>>exp);                         // averag by shifting bit position, LSBs lost 
  }

