// ------------ examples -------------------------------------------------------------------
void demoHexDigit()
 { Serial.println ("Demo/test digit conversion: ");
   for (byte i=0; i<17; i++) { Serial.print(hexDigit(i));}
   Serial.println();
 }
 
// --- ASCII Conversion --- 
 void demoRtnASCIIcode ()
 { Serial.println ("Demo/test ASCII conversion: ");
   byte j=0;
   for (byte i=0; i<255; i++) 
     { Serial.print(rtnASCIIcode(i));
       if (j++ == 31)
         { Serial.println();
           j=0;
         }  
     }
   Serial.println();
}

 
// --- ASCII Table --- (demo multiple references)
/* Of special note in this first function is there are multiple references (names) tied to 
   the same char array. This was done so as to pass the array to the hex conversion function
   without copying the bytes back and forth. The array is declared, allocated and printed 
   using the name "tmp". Pieces of the array are operated on using the name "OutHex". 
   Two points:
   
     1) The second (or more) name is declared as a "pointer" using the "*" 
        dereference pointer operator. Basically this tells the compiler to reserve space for
        an address.
        
     2) That address must be assigned using "&" reference pointer operator. Basically this
        returns the physical memory address of an object. In this case it returns the address 
        of the char array member. Place the "&" in front of the array member reference:
                OutHex= & tmp[3];
        "OutHex" is now considered a char array starting with the third member of the 
         character array "tmp".
        
     Also note that in this function the values to be converted are actually stored in the 
     char array that holds the result. The values are in fact overwritten by their ASCII 
     Hex charters. This is the reason that the conversion routine was written in a manner 
     that starts with the Low nibble. After the High nibble is converted the original 
     source data is destroyed.
 */


void printASCIItable()                         // print out an ASCII table with Hex codes
  // this is included because it is simple to do with the functions in hand
  { char tmp[]= "0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F  ";
    byte i,j,n,l;
    char *OutHex;

    Serial.println("");
    Serial.println("ASCII Table:");
    // ASCII TABLE
    for ( j=2; j<8 ; j++)
      { for ( i=0; i<16 ; i++)                 // fill one row with ASCII characters
        { n=i*3;                               // followed by two spaces
          l= (j * 16)+i;
          tmp[n]=char(l);
          tmp[n+1]=' ';
        } 
        Serial.print("ASCII: ");               // print ASCII string 
        Serial.println(tmp);
        for ( i=0; i<16 ; i++)                 // convert to HEX
        { n=tmp[i*3];                          // note that in this case the number we
          OutHex= & tmp[i*3];                  // are converting is actaully the first
          formatByteAsHex (n, OutHex);         // byte in the string we are passing
        }
        Serial.print("  HEX: ");               // print HEX string 
        Serial.println(tmp);                         
      }                                        // and repeat
  }
  

// --- example one ---   (demo use of formatByteAsHex --- returned string)
void printHexByteOne()                         // example using returned string with print
  { char tmp[]="   ";                          // in this case we send a number and string
    Serial.print("Example One: ");             // the sring is returned to Serial.Print 
    for (byte i=12; i<20; i++)                 // with hex digits
      { Serial.print(formatByteAsHex (i, tmp));
      }
      Serial.println("");                      // output: "0C 0D 0E 0F 10 11 12 13"
  } 

// --- example two ---   (demo use of formatByteAsHex --- passes string)
void printHexByteTwo()                         // example using returned string with print
  { char tmp[]="   ";                          // this is nearly the same but in this case
    Serial.print("Example Two: ");             // we call the format function and then 
    for (byte i=12; i<20; i++)                 // we call the print function. The idea is to
      { formatByteAsHex (i, tmp);              // show it can be used either way.     
        Serial.print ( tmp);
      }
      Serial.println("");                      // output: "0C 0D 0E 0F 10 11 12 13"
  } 

// --- example three ---   (demo use of formatWordAsHex --- returns string)
void printHexWordOne()                         // example using returned string with print
  { char tmp[]="     ";                        // in this case we send a number and string
    word ww = 0xD431;                          // sample word, this could be andress
    Serial.print("Example Three (D431):  ");    // the sring is returned to Serial.Print 
    Serial.println(formatWordAsHex (ww, tmp)); // range of word is 65K
  } 

// --- example four ---   (demo use of formatWordAsHex --- passes string)
void printHexWordTwo()                         // example using passed string with print
  { char tmp[]="     ";                        // in this case we send a number and string
    word ww = 54321;                           // sample word, this could be andress
    Serial.print("Example Four (54321):  ");   // the sring is will be passed Serial.Print 
    formatWordAsHex (ww, tmp);
    Serial.println(tmp);
  } 

// --- example five ---   (demo use of formatWordAsDec --- returns string)
void printDecWordOne()                         // example using returned string with print
  { char tmp[]="     ";                        // in this case we send a number and string
    word ww = 0xD431;                          // sample word, this could be andress
    Serial.print(" Example Five (D431): ");   // the sring is returned to Serial.Print 
    Serial.println(formatWordAsDec (ww, tmp)); 
  } 

// --- example six ---   (demo use of formatWordAsDec --- passes string)
void printDecWordTwo()                         // example using returned string with print
  { char tmp[]="     ";                        // in this case we send a number and string
    word ww = 54321;                           // sample word, this could be andress
    Serial.print(" Example Six (54321): ");    // the sring is will be passed Serial.Print  
    formatWordAsDec (ww, tmp); 
    Serial.println(tmp); 
  } 

// --- example seven ---   (demo/test use of demoFormatRamDump )
void demoFormatRamDump()                         
  { word addr= 54321;
    char buffer[64];
    char data[16]; 
    for(byte i=0; i<16; i++) { data[i]= i + 0x41;}
    Serial.println("Example Seven (formatRamDump):");
    Serial.println(formatRamDump('H', addr, data, buffer));
    Serial.println("Example Seven (formatRamDump/overloaded):");
    Serial.println(formatRamDump(addr, buffer));
    Serial.println();
  }

// --- example eight ---   (demo/test use of overloading )
void demoOverLoading()                         
  { Serial.println("Example Eight (overloading):");
    //          012345678900123456789          // cheat: to figure byte locations
    char tmp[]="A B C D E F G H ";             // Sample Data
    Serial.println(tmp);                       // print sample data  
    for (byte i=7; i>0; i--)                   // with hex digits
      { formatByteAsHex (& tmp[(2*i)]);}       // format each byte except the first
    Serial.println(formatByteAsHex(tmp));      // format and print first byte
    tmp[4]=0;                                  // stuff with null terminator
    tmp[5]=0;                                  // stuff with null terminator
    tmp[0]=0xD4;                               // stuff high byte
    tmp[1]=0x31;                               // stuff low byte
    Serial.print("HEX 0xD431 as HEX: ");
    Serial.println(formatWordAsHex(tmp));  
    tmp[0]=0xD4;                               // stuff high byte
    tmp[1]=0x31;                               // stuff low byte
    Serial.print("HEX 0xD431 as DEC: ");
    Serial.println(formatWordAsDec(tmp));
    Serial.println();
  } 

// --- example nine ---   (EEPROMDump)
void EEPROMDump()
  { char buffer[60];
    word addr=0;
    Serial.println("EEPROM Dump:");
    while (addr < E2END)
      { for (byte i=0; i<16; i++)
          { buffer[i]=EEPROM.read(addr++);
          }
        Serial.println(formatRamDump(addr-16, buffer));          
      }  
  }
  
// -------------------------------------------------------------------------------- 

