/* EEPROM_TempSensor_Calibration_Constants */
/* Stores internal temperature sensor Calibration Constants in EEPROM 
   Uses first 32 bytes of EEPROM for working storage
   Uses last  32 bytes of EEPROM for backup storage    */

#include <EEPROM.h>
//=======================================================================================
//== replace these with the actual values to be used for this specific Arduino board ==//  
//=======================================================================================
word        CovrtOffsetR     = 1387;          // Observed Reading for 72 degrees Fahrenheit
float       Covrt2Celsius    = 0.25;          // calibrated conversion factor for celsius
char        IdString[17]="Your string here";  // fill in your ID string here
//........................1234567890123456    // 16 charateres max
//=======================================================================================
// EEPROM addresses constants
const word EEmask   =  0;                   //  1 byte location of EEPROM storage mode mask
const word EEflag   =  1;                   //  1 byte location of EEPROM storage mode flag
const word EEoffsetR=  2;                   //  2 byte location of CovrtOffset 
const word EEcelsius=  4;                   //  2 byte location of Covrt2Celsius 
const word EEminutes=  6;                   //  2 byte location of Report Target Minutes
const word EEunused0=  8;                   //  2 byte location -- unused --
const word EEunused1= 10;                   //  2 byte location -- unused --
const word EEunused2= 12;                   //  2 byte location -- unused --
const word EEunused3= 14;                   //  2 byte location -- unused --
const word EEidtring= 16;                   // ID string w/o termiantion size (16) 
const word EEidsize = 16;                   // 24 byte location of IdString 
const word EEwdsize = EEidtring+EEidsize;   // Working data storage size      (32)
//----------------------------------------------------------------------------------------
// you may notice some extra space allocated ... these may be used in a later program
//----------------------------------------------------------------------------------------
const word StorageWorking=EEmask;           // EEPROM start for working copy of constants
// EEPROM start for backup  copy of constants 
// note we have to add 1 to the value
// Becasue addresses begin with zero not one
const word StorageBackup =((E2END-(EEwdsize))+1);       
const word CovrtFactor   = 65532;           // The higher this number is the better 

//-----------------------------------------------------------------------------------------
void setup()
 { char WorkStr[EEwdsize+4];                // used to build array to write
   // this array is a bit long so that we can stuff an extra zero byte at the end     
   float tempflt=0;                         // used to convert celsius & fahrenheit factors  
   word  tempwrd=0;                         // used to save celsius & fahrenheit factors 
   float saveflt;                           // used to save conversion factor   
   word i=0;                                // index into work array
   word j=0;                                // counter 
   word k=0;                                // marker
   char c;                                  // one character
     
   Serial.begin(9600);                      // so we can report progress
   // build array to store -------------------------
   // EEmask and EEflag ---------------
   WorkStr[i++]=0xFF;                       // stuff hex FF in array
   WorkStr[i++]=0xFF;                       // stuff hex FF in array
   // CovrtOffset ----------------------------------
   WorkStr[i++]= highByte(CovrtOffsetR);    // stuff CovrtOffset in array
   WorkStr[i++]= lowByte (CovrtOffsetR);
   // Celsius ------------------------------------
   tempwrd =word(Covrt2Celsius*CovrtFactor); // mutiply and drop decimals
   Serial.print   (F("Celcius factor = "));
   Serial.println   (Covrt2Celsius, 5);
   Serial.print   (F("multiplied by "));
   Serial.print     (CovrtFactor);
   Serial.print   (F(" = "));
   Serial.println   (Covrt2Celsius * CovrtFactor, 5);
   Serial.print   (F("Celcius factor stored as Word= "));
   Serial.print     (tempwrd, DEC);
   Serial.print   (F("(HEX: "));
   Serial.print     (tempwrd, HEX);
   Serial.println (F(")"));
   WorkStr[i++]=highByte(tempwrd);              // stuff Celcius factor in array
   WorkStr[i++]=lowByte(tempwrd);
   Serial.println ();
   Serial.flush   ();

   // Minutes between report lines ----------------
   WorkStr[i++]=0x00;                       // stuff hex 00 in array
   WorkStr[i++]=0x01;                       // stuff hex 01 in array

   // Extra space we may use later ----------------
   while (i<EEidtring) {WorkStr[i++]=0xFF;} // stuff hex 00 in array

   // ID String -------------------------------------
   IdString[17]=0;                         // make sure we have a null terminator
   Serial.print   (F("ID String: "));
   // Serial.println   (IdString);          // we are not going to print the
   j=0;c=1; k=i;                            // ID string until we filter it
   while (c!=0)                             // stuff id string in array
    { c=IdString[j++];
      if (c<32)c=0;                         // filter non-printing characters
      WorkStr[i++]=c;
    }
   while (j++ <EEidsize) { WorkStr[i++]=0xFF;} // fill in the rest with ones
   Serial.println (& WorkStr[k]);           // now print it
   Serial.flush();

   
   // EEPROM Write ---------------------------------
   // erase the the entire the EEPROM
   for (i=0 ;i<E2END; i++)
    { if (EEPROM.read (i) != 0xFF)
          EEPROM.write(i,    0xFF);
    }
   
   // copy the array to EEPROM (working copy)
   for (i=0 ;i<EEwdsize; i++)
    { if (EEPROM.read (StorageWorking+i) != WorkStr[i])
          EEPROM.write(StorageWorking+i,    WorkStr[i]);
    }
   // copy the array to EEPROM (backup copy) 
   for (i=0 ;i<EEwdsize; i++)
    { if (EEPROM.read (StorageBackup +i) != WorkStr[i])
          EEPROM.write(StorageBackup +i,    WorkStr[i]);
    }
   Serial.println ();
   Serial.println (F("==============================================="));
   Serial.println ();
   Serial.flush();
  //--------------------------------------------------------------------- 
  // now read it all back
  //--------------------------------------------------------------------- 
   i=EEidtring;j=0; c=1;
   while (c!=0,j< EEidsize)
    {c=EEPROM.read( i++);                   // read the ID string
     WorkStr[j++]=c; 
    } 
  WorkStr[EEidsize]=0;                     // just in case
  Serial.print   (F("ID String read from EEPROM: "));
  Serial.println (WorkStr);                // print it
  Serial.flush();
 
  // Get the Offset; ------------------------------
  tempwrd= EEPROM.read(EEoffsetR)<<8;
  tempwrd= tempwrd + EEPROM.read(EEoffsetR + 1);
  Serial.print   (F("Offset read from EEPROM: "));
  Serial.println   (tempwrd, DEC);        // print it
  Serial.flush();
    
  // Get the celsius factor; ----------------------
  tempwrd= EEPROM.read(EEcelsius)<<8;
  tempwrd= tempwrd + EEPROM.read(EEcelsius + 1);
  Serial.print   (F("Celsius factor read from EEPROM: "));
  Serial.println   (tempwrd, DEC);         // print it
  // now we need to convert it
  tempflt= float(tempwrd)/ CovrtFactor;
  Serial.print   (F("Celsius factor converted back to floating point: "));
  Serial.println   (tempflt, 5);          // print it
  Serial.flush();
 
  Serial.println   ();
  Serial.println (F("Th-th-th-th-th- ... that's all folks!"));
  Serial.flush();
  // http://www.youtube.com/watch?v=-_kwXNVCaxY
  // http://en.wikipedia.org/wiki/Porky_Pig

//    //---//---//----//----
//   Serial.print   (F("--Got Here, i= "));
//   Serial.println   (i, DEC);
//   Serial.println   (j, DEC);
//   Serial.flush();
//   while(true);                           // debugging, stops program
//   //---//---//----//----
}

void loop() {} // do nothing

