/*	Program: Hello World in four languages using PROGMEM */

//---declarations------------------------------------------------------
/* Include function to use flash memory (PROGMEM) for fixed data storage */
#include <avr/pgmspace.h>

/* this is where we define our PROGMEM string */
char HelpStr[] PROGMEM = "E=English, F=French, S=Spanish, I=Italian, A=All";
char EnglishGreetingStr[] PROGMEM = "Hello World";
char FrenchGreetingStr[]  PROGMEM = "Bonjour tout le monde";
char SpanishGreetingStr[] PROGMEM = "Hola mundo";
char ItalianGreetingStr[] PROGMEM = "Ciao mondo";

/* Here we set up an enumerated array of constants.
   This is how all the example code is shown. I was 
   unable to find an example anywhere that put strings 
   in PROMEN did not use an array to access them.      */
const char *Language[] PROGMEM = 
{ EnglishGreetingStr,			             // Language [0]
  FrenchGreetingStr,			             // Language [1]
  SpanishGreetingStr,			             // Language [2]
  ItalianGreetingStr };			             // Language [3] 

/* Here we set up a seperate constant name for each string.
   This may be the only example of this type syntax/usage. */
const char *English PROGMEM = {EnglishGreetingStr};
const char *French PROGMEM = {FrenchGreetingStr};
const char *Spanish PROGMEM = {SpanishGreetingStr};
const char *Italian PROGMEM = {ItalianGreetingStr};
const char *HelpMe PROGMEM = {HelpStr};

/* Unfortunately the software can not use the strings in 
   PROGMEM directly. We have to use special function to
   transfer the strings from PROGMEM to SRAM so that they
   are in teh same memory where the program runs. This
   sets a area in SRAM to work with the strings.       */
char OutBuffer[60];

/* this byte is used to read from the RS232/USB port */
char junk;
         
//---setup------------------------------------------------------
void setup() 
{ /* run once */ 
  Serial.begin(9600); // This sets up a serial connection, 9.6 kbts/s.
  /* wait for serial port to connect. Needed for Leonardo only */
  while (!Serial) { ; }
  /* We are going to prompt the user for input.
     Note that we are using the F() function here to wrap the 
     "string literal"                                          */
  Serial.println (F("Enter: E, F, S, I or A"));
  Serial.println ("");
}

//----main-----------------------------------------------------
void loop() 
{ /* loop until the end of time */
  if (Serial.available() > 0) 
  {
     /* get incoming byte */
     junk = Serial.read();
     /* this next line is useful for debugging if you remove the comment markers   */
     /* Serial.println( junk );  */
     /* now that we have a character from the user we have to decide what
        to do. First off we are really particular. We will only accept
        1 of the 5 charcters we asked for to begin with. If the character
        is not in that group we are going to send the user another message
        to explain the situation. That is the string tha we defined as 
        "HelpMe" in our declarations. So the first thing that we do is copy
        that string to the OutBuffer using the special PROGMEM string copy
        function. I am not going to try to explain the syntax because I
        do not understand it myself. The importent parts to know are:
          1) strcpy_P() is the special function to copy a string from PROGMEM
          2) the first parameter (in thise case "OutBuffer") is the SRAM array
             to copy the string to.
          3) the last parameter (in this case "HelpMe") is the constant that
              points to the string that we wish to retrieve.                        */
     strcpy_P(OutBuffer, (char*)pgm_read_word(&(HelpMe)));
     
     /* Now to test our user input. This is very straight foward. If the byte
        matches the character then we copy the appropriate string to the
        OutBuffer using the same syntax that we used for default string above.
        We repeat the test for each character (fall through logic).                  */ 
     if (junk == 'E')
       { strcpy_P(OutBuffer, (char*)pgm_read_word(&(English)));}
     if (junk == 'F')
       { strcpy_P(OutBuffer, (char*)pgm_read_word(&(French)));}
     if (junk == 'S')
       { strcpy_P(OutBuffer, (char*)pgm_read_word(&(Spanish)));}
     if (junk == 'I')
       { strcpy_P(OutBuffer, (char*)pgm_read_word(&(Italian)));}
     if (junk == 'A')
        /* This is a bit different. Here we are using the Constant array to print
           all the various language strings that we defined. This is maily here to
           show the difference in systax: speciically the "[i]" at for the 
           constant array. The loop starts with the variable i equal to zero.
           It copies the PROGMEM string to the OutBuffer and prints it. Then the 
           loop increases the value of i by 1 (i++ means i=i+1) and checks to see
           if the value is less than 4. If it is then then the loop is processes
           again. It repats this process until i=4.                                   */ 
       { for (int i = 0; i < 4;  i++)
           { strcpy_P(OutBuffer, (char*)pgm_read_word(&(Language[i])));
             Serial.println( OutBuffer );
           }
         /* clear the buffer so that the main print statement will output a blank line 
            C use schar(0) to define the end of a string. 
            We simply write 0 to the first byte          */  
         OutBuffer[0]= char(0) ;  
       }
     /* Print whatever we happen to have in the OutBuffer */
     Serial.println( OutBuffer );
  }
}

